##################################################
# MonitorMSVS.ps1
# Used to monitor Microsoft Virtual Server 2005.
# 
# Created: 12/01/2006
# Author: Tyson Kopczynski
##################################################
param([string] $ServerName = $(throw write-host `
    "Please specify the name of the MSVS host to monitor!" `
    -Foregroundcolor Red))

##################################################
# Main
##################################################
$URL = "http://$($ServerName):1024/VirtualServer/VSWebApp.exe?view=1"

#--------------------
# Begin Script
#--------------------
write-host "----------------------------------------"
write-host "-            MonitorMSVS               -"
write-host "----------------------------------------"
write-host   
write-host "Checking MSVS Status" -NoNewLine

.{
    trap{write-host `t "[ERROR]" -Foregroundcolor Red;
        throw write-host $_ -Foregroundcolor Red;
	Break}

    $Ping = new-object Net.NetworkInformation.Ping
    $Result = $Ping.Send($ServerName)

    if ($Result.Status -eq "Success"){
        write-host `t "[ONLINE]" -Foregroundcolor Green
        }
    else{
        write-host `t "[OFFLINE]" -Foregroundcolor Red
        write-host
        Break
        }
}

#--------------------
# Get list of VMs
#--------------------    
$Webclient = new-object Net.WebClient
$Webclient.UseDefaultCredentials = $True

write-host "Getting VM Names" -NoNewLine

.{
    trap{write-host `t "[ERROR]" -Foregroundcolor Red; 
        throw write-host $_ -Foregroundcolor Red;
        Break}
        
    $Data = $Webclient.DownloadString("$URL")
    
    write-host `t "[DONE]" -Foregroundcolor Green
}

# This Regex gets a list of server entries from the data returned
$Servers = [Regex]::Matches($Data, '(?<=&amp;vm=)[^"\r\n]*(?=" )')

# There are many duplicates so you need to group them
# Plus, this gives you a better name for your property
$Servers = $Servers | group Value | select Name

#--------------------
# Get VM data
#--------------------    
write-host "Getting VM Data" -NoNewLine

.{
    trap{write-host `t`t "[ERROR]" -Foregroundcolor Red; 
        throw write-host $_ -Foregroundcolor Red;
        Break}
        
    $VSMachines = get-wmiobject -namespace "root/vm/virtualserver" `
        -class VirtualMachine -computername $ServerName `
        -ErrorAction Stop
    
    write-host `t`t "[DONE]" -Foregroundcolor Green
}

foreach ($Server in $Servers){
    &{
        $VSMachine = $VSMachines | where {$_.Name -eq $Server.Name}

        if($VSMachine){
            $Uptime = $VSMachine.Uptime / 60
            $Memory =  ($VSMachine.PhysicalMemoryAllocated / 1024) / 1000
            $Disk = ($VSMachine.DiskSpaceUsed / 1024) /1000
            
            add-member -inputObject $Server -membertype noteProperty `
                -name "Status" -value "Online"
            add-member -inputObject $Server -membertype noteProperty `
                -name "Uptime" -value $Uptime
            add-member -inputObject $Server -membertype noteProperty `
                -name "CPU" -value $VSMachine.CpuUtilization
            add-member -inputObject $Server -membertype noteProperty `
                -name "Memory" -value $Memory
            add-member -inputObject $Server -membertype noteProperty `
                -name "Disk" -value $Disk
            }
        else{
            add-member -inputObject $Server -membertype noteProperty `
                -name "Status" -value "Offline"
            }
    }
    }

$Servers | format-table Name, Status `
    ,@{label="Uptime Mins"; expression={"{0:N0}" -f $_.Uptime}} `
    ,@{label="CPU %"; expression={$_.CPU}} `
    ,@{label="Memory MB"; expression={"{0:N0}" -f $_.Memory}} `
    ,@{label="Disk MB"; expression={"{0:N0}" -f $_.Disk}} `
    -wrap
# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUA2kwEAShwkP5i5Zuzr0Dbs0c
# UBKgggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBSqwkH/FngGpqugPYPRykpFV1YI6zANBgkqhkiG9w0BAQEFAASCAQDO
# 3fT2afzOJq25D/3XhiVD7lTUqYvBBYCLlgB2GSdmcCBqUttEsDstWbh3yFsz4zUd
# Iw21d6JRUYiaDzld8a+bgEaWdVF/i7ZNl18UrBpq6d572tfY+ce1tJ5g9sO/pXDw
# ZgQDyYdRJNhOTvOmlRx6yG7LHdpCctV++o58Kh+lNMMFnd3PK8hlowf0hH2iqJdk
# JFgeiUtkxo+RaVk9wi4iGyHDvX28rD9eyBXApxMkd8BdGhdm/E8yn4VqtZ41vC0G
# QRBsTGMRhqg4xnTHNF4u4WsNfBIsVE7XqZwSLVWyY0jgDkVkHWLkLWEaaM1Rmmgs
# KPUmaOZ4RRNQTk04J9ne
# SIG # End signature block
