﻿##################################################
# GetDatabaseSizeReport.ps1
# Used to generate an Exchange database size report.
# 
# Created: 10/26/2006
# Author: Tyson Kopczynski
##################################################
param([string] $ExportFile)

##################################################
# Functions
##################################################
#-------------------------------------------------
# Export-DataTable
#-------------------------------------------------
# Usage:        Used to export a DataSet to a CSV file.
# $Data:        A DataSet object.
# $FileName:    The name of the export CSV file.

function Export-DataTable{
    param ($Data, $FileName)

    $Null =
        [System.Reflection.Assembly]::LoadWithPartialName( `
            "System.Windows.Forms")
    
    trap{write-host "[ERROR] $_" -Foregroundcolor Red; Continue}

    if ($FileName -eq ""){
        $exFileName = new-object System.Windows.Forms.saveFileDialog
        $exFileName.DefaultExt = "csv"
        $exFileName.Filter = "CSV (Comma delimited)(*.csv)|*.csv"
        $exFileName.ShowHelp = $True
		$exFileName.ShowDialog()

        $FileName = $exFileName.FileName
        }

    if ($FileName -ne ""){
        $LogFile = new-object System.IO.StreamWriter($FileName, $False)

         for ($i=0; $i -le $Data.Columns.Count-1; $i++){
            $LogFile.Write($Data.Columns[$i].ColumnName)
            
            if ($i -lt $Data.Columns.Count-1){
                $LogFile.Write(",")
                }
            }
        
        $LogFile.WriteLine()

        foreach ($Row in $Data.Rows){
            for ($i=0; $i -le $Data.Columns.Count-1; $i++){
                $LogFile.Write($Row[$i].ToString())
                
                if ($i -lt $Data.Columns.Count-1){
                    $LogFile.Write(",")
                    }
                }
                
            $LogFile.WriteLine()
            }
        
        $LogFile.Close()
        }
    }

##################################################
# Main
##################################################
#--------------------
# Load Libraries
#--------------------
. .\LibraryGen.ps1

#--------------------
# Set Config Vars
#--------------------
$ScriptName = "GetDatabaseSizeReport.ps1"
$ScriptUsage = "Used to generate an Exchange database size report."
$ScriptCommand = "$ScriptName -ExportFile value"
$ScriptParams = "ExportFile = The export CSV file path/filename."
$ScriptExamples = "$ScriptName ""report.csv"""
$ErrorLogName = "GetDatabaseSizeReport.log"
$Date = Date

#--------------------
# Verify Required Parameters
#--------------------
if ($args[0] -match '-(\?|(h|(help)))'){
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

#--------------------
# Define DataTables
#--------------------
$ServersTable = new-object System.Data.DataTable
$ServersTable.TableName = "Servers"
[Void]$ServersTable.Columns.Add("Name")
[Void]$ServersTable.Columns.Add("Status")

$ReportTable = new-object System.Data.DataTable
$ReportTable.TableName = "Servers"
[Void]$ReportTable.Columns.Add("Server")
[Void]$ReportTable.Columns.Add("Database")
[Void]$ReportTable.Columns.Add("Drive")
[Void]$ReportTable.Columns.Add("FreeSpace")
[Void]$ReportTable.Columns.Add("Size")

#--------------------
# Begin Script
#--------------------
# Setup ErrorLog
$ScriptName + " Ran on: " + $Date | out-file $ErrorLogName

write-host
Get-ScriptHeader $ScriptName $ScriptUsage
write-host

#--------------------
# Get Servers and Status
#--------------------
write-host "Getting Mailbox Servers" -NoNewLine
$MailboxServers = get-mailboxserver
write-host `t "[DONE]" -Foregroundcolor Green

write-host "Getting Status Info" -NoNewLine

$MailboxServers | foreach-object -Begin {$i=0;} `
    -Process {&{$Ping = new-object Net.NetworkInformation.Ping;
            $MBServerName = $_.Name;
            trap{"Ping [ERROR]: " + $MBServerName + " $_" | out-file `
                $ErrorLogName -Append; Continue};
            $Result = $Ping.Send($MBServerName);
            if ($Result.Status -eq "Success"){  `
                [Void]$ServersTable.Rows.Add($MBServerName, "Online")} `
            else{[Void]$ServersTable.Rows.Add($MBServerName, "Offline")};
            $i = $i+1;
            write-progress -Activity "Pinging Servers - $($MBServerName)" `
                -Status "Progress:" `
                -PercentComplete ($i / $MailboxServers.Count * 100)}}

write-host `t "[DONE]" -Foregroundcolor Green

# Write status info to ErrorLog
$ServersTable | out-file $ErrorLogName -Append

#--------------------
# Get Report Info
#--------------------
write-host "Getting Report Info"  -NoNewLine

$OnlineServers = $ServersTable.Select("Status = 'Online'")

foreach ($Server in $OnlineServers) {
    &{
        trap{"Make Report [Error]: " + $Server.Name + " $_" | `
            out-file $ErrorLogName -Append; Continue}

        write-progress -Activity "Getting Database Info - $($Server.Name)" `
            -Status "Stand by..."    
      
        $Databases = get-mailboxdatabase -Server $Server.Name | `
            select Name, StorageGroupName, EdbFilePath

        foreach ($Database in $Databases){
            &{
                write-progress `
                    -Activity "Getting Drive Info - $($Server.Name)" `
                    -Status "Stand by..."
                
                $DBDriveName = $Database.EdbFilePath.DriveName
                $DBDrive = `
                    get-wmiobject Win32_PerfRawData_PerfDisk_LogicalDisk `
                    -Computer $Server.Name -Filter "Name = '$DBDriveName'"
                  
                write-progress -Activity `
                    "Getting Drive Size Info - $($Server.Name)" `
                    -Status "Stand by..."        
                
                # Needed to replace \ with \\
                $DBPath = $Database.EdbFilePath.PathName.Replace("\","\\")
                $DBFile = get-wmiobject CIM_DataFile -Computer $Server.Name `
                    -Filter "Name = '$DBPath'"
                  
                $DBName = $Database.StorageGroupName + "\" + $Database.Name 
                
                # Needed to convert from MB to GB
                $DBDriveFreeSpace = $DBDrive.FreeMegabytes / 1000      
                
                # Needed to convert Bytes to GB
                $DBSize = $DBFile.FileSize / 1073741824

                [Void]$ReportTable.Rows.Add($Server.Name, $DBName, `
                    $DBDriveName, $DBDriveFreeSpace, $DBSize) 
            }
            }
        
        write-progress -Activity `
            "Getting Database Info - $($Server.Name)" `
            -Status "Done" -completed $True
    }
    }

write-host `t "[DONE]" -Foregroundcolor Green

$ReportTable | format-table -groupBy Server Database, Drive, `
    FreeSpace, Size -autosize 

$Null = Export-DataTable $ReportTable $ExportFile

write-host
write-host "Script is now DONE!" -Foregroundcolor Green
write-host "Check the $ErrorLogName for errors." -Foregroundcolor Yellow 
# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU9zcOasfN08LgCLNyg2wlU2N1
# 0+mgggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBSlOJ11DzdjyNRrM0jpWgrUOmmZUTANBgkqhkiG9w0BAQEFAASCAQCY
# L+HgTz5mMOacM0gv0rypq/YzQ8RiFFQeQdnR43Br9kRTriQL8ITqtgNm2AblTf+M
# Uu2bkDX3VnKEOWbzp14mhK0E67s6ySXUzIpKUTL7R62rpXGhS9GTlMrGPQDlH4+6
# +8M+7tuCx9y6pgRTD2rju+j1t185oTtVXdJXQlAHgL+9YCSn8HGan2guGyKNoyf/
# vHy1grJae9ND92rXtkEhlUsf9HAOkXB7pWWksYPd5mDwG09M8ikGfbl6XTQHh+N/
# p7F0Hi6eSGtwy9MnT2unHxSCIeiplwcEyLeX2ZumoKLBvECl3lWXvbhQIlxgUQ3N
# opiXnSe0EN/udR4d/Gay
# SIG # End signature block
