##################################################
# ProvisionWebFolders.ps1
# Used to provision new user Web folders.
# 
# Created: 9/12/2006
# Author: Tyson Kopczynski
##################################################
param([string] $TemplatePath = $(throw write-host `
    "Please specify the source template path of the folder structure to" `
    "be copied." -Foregroundcolor Red), [string] $ImportFile = $(throw `
    write-host "Please specify the import CSV file name." `
    -Foregroundcolor Red))

##################################################
# Functions
##################################################
#-------------------------------------------------
# Clear-Inherit
#-------------------------------------------------
# Usage:        Used to protect against inherited access rules
#               and remove all inherited explicitly defined rules.
# $Object:      The directory or file path.  ("c:\myfolder" or 
#               "c:\myfile.txt")

function Clear-Inherit{
    param ($Object)

    $SD = get-acl $Object
    $SD.SetAccessRuleProtection($True, $False)
    set-acl $Object $SD
    }

#-------------------------------------------------
# Set-Owner
#-------------------------------------------------
# Usage:        Used to set the owner on a folder or file.
# $Object:      The directory or file path.  ("c:\myfolder" or 
#               "c:\myfile.txt")
# $Identity:    User or Group name. ("Administrators" or
#               "mydomain\user1"

function Set-Owner{
    param ($Object,
        [System.Security.Principal.NTAccount]$Identity)

    # Get the item that will be changed
    $Item = get-item $Object

    # Set the owner
    $SD = $Item.GetAccessControl()
    $SD.SetOwner($Identity)
    $Item.SetAccessControl($SD)
    }

#-------------------------------------------------
# Clear-SD
#-------------------------------------------------
# Usage:        Used to drop all permissions on a folder or file.
# $Object:      The directory or file path.  ("c:\myfolder" or 
#               "c:\myfile.txt")

function Clear-SD{
    param ($Object)

    # Get the security descriptor for the object
    $SD = get-acl $Object

    # Set the SD to Everyone - Full Control
    #
    # Yes this isn't a best practice, if you don't like it then
    # set the SD to the current user. 
    $SD.SetSecurityDescriptorSddlForm("D:PAI(A;OICI;FA;;;WD)")
    set-acl $Object $SD
    }

#-------------------------------------------------
# Add-ACE
#-------------------------------------------------
# Usage:        Grants rights to a folder or file.
# $Object:      The directory or file path.  ("c:\myfolder" or 
#               "c:\myfile.txt")
# $Identity:    User or Group name. ("Administrators" or
#               "mydomain\user1"
# $AccessMask:  The access rights to use when creating the access rule.
#               ("FullControl", "ReadAndExecute, Write", etc.)
# $Type:        Allow or Deny access. ("Allow" or "Deny")

function Add-ACE{
    param ($Object,
    [System.Security.Principal.NTAccount]$Identity,
    [System.Security.AccessControl.FileSystemRights]$AccessMask, 
    [System.Security.AccessControl.AccessControlType]$Type)

    $InheritanceFlags = `
        [System.Security.AccessControl.InheritanceFlags]`
        "ContainerInherit, ObjectInherit"
    $PropagationFlags = `
        [System.Security.AccessControl.PropagationFlags]"None"

    # Get the security descriptor for the object
    $SD = get-acl $Object

    # Add the AccessRule
    $Rule = new-object `
        System.Security.AccessControl.FileSystemAccessRule($Identity, `
        $AccessMask, $InheritanceFlags, $PropagationFlags, $Type)
    
    $SD.AddAccessRule($Rule)
    set-acl $Object $SD
    }

#-------------------------------------------------
# Remove-ACE
#-------------------------------------------------
# Usage:        Removes rights to a folder or file.
# $Object:      The directory or file path.  ("c:\myfolder" or 
#               "c:\myfile.txt")
# $Identity:    User or Group name. ("Administrators" or
#               "mydomain\user1"
# $AccessMask:  The access rights to use when creating the access rule.
#               ("FullControl", "ReadAndExecute, Write", etc.)
# $Type:        Allow or Deny access. ("Allow" or "Deny")

function Remove-ACE{
    param ($Object,
        [System.Security.Principal.NTAccount]$Identity,
        [System.Security.AccessControl.FileSystemRights]$AccessMask, 
        [System.Security.AccessControl.AccessControlType]$Type)

    # Get the security descriptor for the object
    $SD = get-acl $Object

    # Remove the AccessRule
    $Rule = new-object `
        System.Security.AccessControl.FileSystemAccessRule($Identity, `
        $AccessMask, $Type)
    $SD.RemoveAccessRule($Rule)
    set-acl $Object $SD
    }

##################################################
# Main
##################################################
write-host "----------------------------------------"
write-host "-         ProvisionWebFolders          -"
write-host "----------------------------------------"
write-host   
write-host "Checking Template Path" -NoNewLine

if (!(test-path $TemplatePath -pathType container)){
    throw write-host `t "$TemplatePath is not a valid directory!" `
        -Foregroundcolor Red
    }
else {
    write-host `t "[OK]" -Foregroundcolor Green
    }

write-host "Checking Import File" -NoNewLine

if (!(test-path $ImportFile -pathType leaf)){
    throw write-host `t "$ImportFile is not a valid file!" -Foregroundcolor Red
    }
else {
    write-host `t "[OK]" -Foregroundcolor Green
    }

#--------------------
# Set Vars
#--------------------
$Owner = "Administrators"
$Targets = import-csv $ImportFile

#--------------------
# Provision Web Folders
#--------------------
write-host
write-host "Provision Web Folders:"

foreach ($Target in $Targets){
    $Path = join-path $Target.DestPath $Target.UserName
    $UserName = $Target.UserName
    
    write-host $Path
   
    if (!(test-path $Path)){
        copy-item $TemplatePath -Destination $Path -Recurse `
        -ErrorVariable Err -ErrorAction SilentlyContinue

        if (!$Err){
            write-host " Folder " -NoNewLine
            write-host "[COPIED]" -Foregroundcolor Green
            
            # Used to stop loops
            $Err = $False
            
            .{
                trap{write-host "[ERROR] Failed to take ownership!" `
                    -Foregroundcolor Red; 
                    $Script:Err = $True;
                    Continue}
                
                # Set Owner
                write-host " SetOwner for $Owner " -NoNewLine
            
                Set-Owner $Path $Owner
                
                if ($Err -eq $False){
                    $Items = get-childitem $Path -Recurse
                    [void]($Items | foreach-object `
                            {Set-Owner $_.FullName $Owner})
                    }
                else{
                    # Stop the loop
                    Continue
                    }

                write-host "[OK]" -Foregroundcolor Green
            }

            .{
                trap{write-host "[ERROR] Failed to add rights!" `
                    -Foregroundcolor Red; 
                    $Script:Err = $True;
                    Continue}
                
                # Add Administrators
                write-host " AddACE for Administrators " -NoNewLine

                Add-ACE $Path "Administrators" "FullControl" "Allow"
                
                if ($Err -eq $False){
                    write-host "[OK]" -Foregroundcolor Green
                    }
                else{
                    # Stop the loop
                    Continue
                    }
            }
                
            .{
                trap{write-host "[ERROR] Failed to clear inherited"`
                    "permissions!" -Foregroundcolor Red;
                    $Script:Err = $True;
                    Continue}
           
                # Clear inherited permissions
                write-host " ClearInherit " -NoNewLine               
                
                Clear-Inherit $Path
                
                if ($Err -eq $False){
                    write-host "[OK]" -Foregroundcolor Green
                    }
                else{
                    # Stop the loop
                    Continue
                    }
            }

            .{
                trap{write-host "[ERROR] Failed to add rights!" `
                -Foregroundcolor Red;
                $Script:Err = $True;
                Continue}

                # Add SYSTEM
                $Err = $False
                write-host " AddACE for SYSTEM " -NoNewLine

                Add-ACE $Path "SYSTEM" "FullControl" "Allow"

                if ($Err -eq $False){
                    write-host "[OK]" -Foregroundcolor Green
                    }
            }

            .{
                trap{write-host "[ERROR] Failed to add rights!" `
                -Foregroundcolor Red;
                $Script:Err = $True;
                Continue}

                # Add User
                $Err = $False
                write-host " AddACE for $UserName " -NoNewLine

                Add-ACE $Path $UserName "FullControl" "Allow"

                if ($Err -eq $False){
                    write-host "[OK]" -Foregroundcolor Green
                    }
            }               
            }
        else {
            write-host " Folder " -NoNewLine
            write-host "Error:" $Err -Foregroundcolor Red
            }
        }
    else {
        write-host " Folder " -NoNewLine
        write-host "[EXISTS]" -Foregroundcolor Yellow        
        }
     
    write-host
    }
       
write-host "Done Provisioning Web Folders:"


# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUFyPsJC84dApXU6gMSZB/g2Ya
# Sk6gggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBSdZP97/u/2Za/pdlslJjW9J04+HzANBgkqhkiG9w0BAQEFAASCAQCk
# 7mmfY1pZpkjzHqL6yqfXLSEm0472HoFeV4QDlQjeTBb5CqpL3LQcmuiWWFsfp5E2
# R9Ld2HFNFZA1kq74SesVGiPplRpW2K1FgqMu0TYB9F1KCsBKJE+7URYiysVp5cqz
# mnA39dt4IcqBYhEaKLen3MiaToy2UX2VGp6J02rufbNIZZNGXtHpbJeBq8fB/gqw
# wyaVQ62oOtRNpH9LiHg91OBiDARsgweN06iY7CPKDDHxdSvlmebT4zQjx6TT4Q2V
# Cl904o82rfZ8mrHWephqkgP9QOL/FdJMaOBv5JwJz3Ck3/OzPNRH+9UrD0s6N02i
# Uc73r+dj0GLLmP3uOlRO
# SIG # End signature block
